import greenfoot.*;  // (World, Actor, GreenfootImage, and Greenfoot)

import java.util.List;
import java.util.ArrayList;
import java.awt.Color;
import java.awt.Font;
//import java.util.Calendar;

/**
 * Un serpent avance et grandi, jusqu'à ce qu'il touche un bord ou se rentre dedans.
 * 
 * @author Bernard Gisin
 * @version 0.1
 */
public class cAct01 extends Actor
{
    private static final int NORTH = 1;
    private static final int EAST  = 2;
    private static final int SOUTH = 3;
    private static final int WEST  = 4;

    private int direction = EAST;
    private int dirKey    =    0; // pour mémoriser les touches de directions.
    private int nPosKey    =   1; // pour mémoriser la position de touche de direction.
    private int nDdirKeyLast = 0; // dernière direction demandée
    
    int nbDeplacement = 0; // nombre de déplacements effectués    

//    cAct02 act02;
    
    public cAct01()
    {
       nbDeplacement = 0;
//       act02 = ((MyWorld)getWorld()).act02;
    }

    /**
     * act()  est appelé plusieurs fois par secondes.
     * C'est a travers lui que les événements sont testés.
     */
    public void act()
    {
        processKeys();

        testTurn(); // tourne dans une direction si une touche a été pressée.
        
        if(canMove()) {
            move();
        }
        else {
            turnLeft();
        }
    }
    
    /**
     * tourne en fonction des touches pressées
     */
    private void processKeys() {
        int nDir = 0;
        
        if (dirKey > 99999999) return;
        
        if(Greenfoot.isKeyDown("up")) {
            nDir = NORTH;
        } else if(Greenfoot.isKeyDown("right")) {
            nDir = EAST;
        } else if(Greenfoot.isKeyDown("down")) {
            nDir = SOUTH;
        } else if(Greenfoot.isKeyDown("left")) {
            nDir = WEST;
        }    

        if (nDir == 0) return; // touche n'est pas une direction.
        if (nDir == nDdirKeyLast) return; // pas deux fois la même direction.

        nDdirKeyLast = nDir; // mémorise la dernière direction demandée
        dirKey = dirKey + nPosKey * nDir; // mémorise la nouvelle direction à venir.
        nPosKey *= 10;
    }

    /**
     * C'est terminé.
     */
    public void Termine()
    {
       Greenfoot.delay(10); // attend l'équivalent de 10 appels à "act()"
       Greenfoot.playSound("BravoTermine.wav");  
       makeImage("Terminé", "Nombre de déplacements : ", nbDeplacement);
       Greenfoot.stop();
 
       // recommence a changer plus...
       //leavesEaten = 0;
       //nbDeplacement = 0;
       //((MangeWorld)getWorld()).populate();
 
    }

    /**
     * Make the score board image.
     */
    private void makeImage(String title, String prefix, int score)
    {
        int WIDTH = 400;
        int HEIGHT = 300;
        float FONT_SIZE = 24.0f;

        setRotation(0);
        setLocation(3, 3);
        GreenfootImage image = new GreenfootImage(WIDTH, HEIGHT);

        image.setColor(new Color(0, 0, 0, 160));
        image.fillRect(0, 0, WIDTH, HEIGHT);
        image.setColor(new Color(255, 255, 255, 100));
        image.fillRect(5, 5, WIDTH-10, HEIGHT-10);
        Font font = image.getFont();
        font = font.deriveFont(FONT_SIZE);
        image.setFont(font);
        image.setColor(Color.WHITE);
        image.drawString(title, 30, 100);
        image.drawString(prefix + score, 30, 200);
        setImage(image);
    }
    
    /**
     * Avance l'ours dans la direction courante.
     */
    public void move()
    {
        if (!canMove()) {
            return;
        }
        switch(direction) {
            case SOUTH :
                setLocation(getX(), getY() + 1);
                break;
            case EAST :
                setLocation(getX() + 1, getY());
                break;
            case NORTH :
                setLocation(getX(), getY() - 1);
                break;
            case WEST :
                setLocation(getX() - 1, getY());
                break;
        }
                
        ((MyWorld)getWorld()).act02.move(direction);

        nbDeplacement +=1;
        // nbDeplacement = dirKey;  // Aide au dépannage.
    }

    /**
     * Test si l'ours peut se déplacer.
     * S'il le peut, returne true sinon retourne false.
     */
    public boolean canMove()
    {
        World myWorld = getWorld();
        int x = getX();
        int y = getY();
        switch(direction) {
            case SOUTH :
                y++;
                break;
            case EAST :
                x++;
                break;
            case NORTH :
                y--;
                break;
            case WEST :
                x--;
                break;
        }
        
        // test for outside border
//        if (x+10 >= myWorld.getWidth() || y+70 >= myWorld.getHeight()) {
        if (x > 470 || y > 470) {
            return false;
        }
        else if (x < 10 || y < 10) {
            return false;
        }
        return true;
    }

    /**
     * Turns towards the left.
     */
    public void turnLeft()
    {
        switch(direction) {
            case SOUTH :
                direction = EAST;
                break;
            case EAST :
                direction = NORTH;
                break;
            case NORTH :
                direction = WEST;
                break;
            case WEST :
                direction = SOUTH;
                break;
        }
    }
    
    /**
     * Tourne dans la direction désirée si une touche a été pressée.
     */
    public void testTurn()
    {
        if ((dirKey == 0) || (((getX()+10) % 20) != 0) || (((getY()+10) % 20) != 0)) return;
        
        direction = dirKey % 10; // nouvelle direction.
        
        dirKey  = dirKey  / 10; // élimine le chiffre des unités.
        nPosKey = nPosKey / 10;
    }

}